<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\MAN\Utilities\Util;

class RelCampFideCashbackController extends CrudWithParameterBag {
    
    protected $dataSourceName = 'relCashback_bonusGerado';
    protected $relCampFideCashbackService;
    protected $util;
    protected $parameterBagColumns = array('DTINICIO', 'DTFIM', 'PERIODO', 'TIPORELATORIO');
    
    public function __construct(Manager $dataSourceManager, ParameterBag $parameterBag, $RelCampFideCashbackService, Util $util) {
        $this->relCampFideCashbackService = $RelCampFideCashbackService;
        $this->util                     = $util;
        parent::__construct($dataSourceManager, $parameterBag);
    }
    
    public function fixConditions (DTO\Request\Filter $request) {
        $conditions = $this->util->getParamsAsArray($request);
        
        if (isset($conditions['PERIODO'])) {
            $conditions['DTINICIO'] = $conditions['PERIODO'][0] ?? $conditions['PERIODO']['start'];
            $conditions['DTFIM'] = $conditions['PERIODO'][1] ?? $conditions['PERIODO']['end'];
        }
        else {
            $conditions['DTINICIO'] = date("d/m/Y");
            $conditions['DTFIM'] = date("d/m/Y");
        }
        
        // implode caixa e venda
        if (isset($conditions['CDCAIXA']) && is_array($conditions["CDCAIXA"])){
            $conditions['CDCAIXA'] = implode(",", $conditions['CDCAIXA']);
        }
        if (isset($conditions['NRSEQVENDA']) && is_array($conditions["NRSEQVENDA"])){
            $conditions['NRSEQVENDA'] = implode(",", $conditions['NRSEQVENDA']);
        }
        
        return $conditions;
    }
    
    public function findSales (DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $this->fixConditions($request);
        $result = $this->relCampFideCashbackService->findSales($conditions);
        $response->addDataSet(new DataSet('relCashback_findSales', $result));
    }
    
    public function findBonusGerado(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->fixConditions($request);
        $result = $this->relCampFideCashbackService->findBonusGerado($conditions);
        $response->addDataSet(new DataSet('relCashback_bonusGerado', $result));
    }
    
    public function findBonusResgatado(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->fixConditions($request);
        $result = $this->relCampFideCashbackService->findBonusResgatado($conditions);
        $response->addDataSet(new DataSet('relCashback_bonusResgatado', $result));
    }
    
    public function findBonusPerdido(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->fixConditions($request);
        $result = $this->relCampFideCashbackService->findBonusPerdido($conditions);
        $response->addDataSet(new DataSet('relCashback_bonusPerdido', $result));
    }
    
    public function findBonusVencer(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->fixConditions($request);
        $result = $this->relCampFideCashbackService->findBonusVencer($conditions);
        $response->addDataSet(new DataSet('relCashback_bonusVencer', $result));
    }
    
    public function findBonusTotal(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->fixConditions($request);
        $result = $this->relCampFideCashbackService->findBonusTotal($conditions);
        $response->addDataSet(new DataSet('relCashback_bonusTotal', $result));
    }
    
    public function findTotalizadores(DTO\Request $request, DTO\Response $response) {
        $conditions = $this->fixConditions($request);
        // $conditions = $conditions['filtro'] ?? null;

        if (isset($conditions)) {
            
            $result = [];
            $result["geradoTotal"] = $this->relCampFideCashbackService->findGeradoTotal($conditions);
            $result["resgatadoTotal"] = $this->relCampFideCashbackService->findResgatadoTotal($conditions);
            $result["perdidoTotal"] = $this->relCampFideCashbackService->findPerdidoTotal($conditions);
            $result["vencerTotal"] = $this->relCampFideCashbackService->findVencerTotal($conditions);
            $result["consumidorTotal"] = $this->relCampFideCashbackService->findTotalConsumidor($conditions);
            
            $response->addDataSet(new DataSet('relCashback_findTotalizadores', $result));
        }
    }
}
