<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Services;

use League\OAuth2\Client\Provider\Google;
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\SMTP;
use PHPMailer\PHPMailer\OAuth;
use PHPMailer\PHPMailer\Exception;


class SMTPTest extends CrudWithParameterBag {

    public function __construct(ParameterBag $parameterBag, $environment, Services\SMTPTokenService $SMTPTokenService) {
        $this->parameterBag                     = $parameterBag;
        $this->environment                      = $environment;
        $this->SMTPTokenService = $SMTPTokenService;
    }

    public function sendEmail(DTO\Request $request, DTO\Response $response){
        date_default_timezone_set('Etc/UTC');
        $row = $request->getRow();
        
        $email = $row['DSEMAILAUVND'];
        $clientId = $row['DSCLIENTID'];
        $clientSecret = $row['DSCLIENTSECRET'];
        
        $token = $this->SMTPTokenService->getToken();
        $refreshToken = $token[0]['DSREFTOKENGOOGLE'];

        try {
            $mail = new PHPMailer(true);
            
            $mail->isSMTP();
            $mail->SMTPDebug = SMTP::DEBUG_OFF;
            $mail->Host = 'smtp.gmail.com';
            
            $mail->Port = 465;
            $mail->SMTPSecure = PHPMailer::ENCRYPTION_SMTPS;
            
            $mail->SMTPAuth = true;
            $mail->AuthType = 'XOAUTH2';
    
    
            $provider = new Google(
                [
                    'clientId' => $clientId,
                    'clientSecret' => $clientSecret,
                ]
            );
            
            $mail->setOAuth(
                new OAuth(
                    [
                        'provider' => $provider,
                        'clientId' => $clientId,
                        'clientSecret' => $clientSecret,
                        'refreshToken' => $refreshToken,
                        'userName' => $email,
                    ]
                )
            );
            $mail->setFrom($email);
            
            $EMAIL = $row['email'];
            $mail->addAddress($EMAIL);
            
            $subject  = 'Teste SMTP - Manage Teknisa';
            $sender = 'Manage - Odhen';
            $content =
                '<p>Teste de configuração de SMTP do Odhen Manage.</p>'.
                '<p>Se não requisitou este teste, ignore essa mensagem.</p>'.
                '<p>Email enviado para: '. $EMAIL . '<br>';
            
            $message    = self::generateEmail($subject, $content);
            
            $mail->CharSet = PHPMailer::CHARSET_UTF8;
            
            $mail->IsHTML(true);
            $mail->Encoding = "base64";
            $mail->Body = $message;
            $mail->Subject = $subject;
            
            
            if($mail->send() != 1)
            {
                $response->setStatus(Response::STATUS_WARNING);
                $response->addMessage(new Message(SMTPManagement::DEFAULT_ERROR_MSG));
                $response->addMessage(new Message($mail->ErrorInfo));
            }
            else{
                $response->setStatus(Response::STATUS_SUCCESS);
                $response->addMessage(new Message(SMTPManagement::DEFAULT_SUCCESS_MSG));
            }
            
        }
        catch (Exception $e) {
            $message = $e->getMessage();
            $response->setStatus(Response::STATUS_WARNING);
            $response->addMessage(new Message(SMTPManagement::DEFAULT_ERROR_MSG));
            $response->addMessage(new Message($message));
        }
    }
    
    
    public function generateEmail($subject, $content) {
    	return
    		'<!DOCTYPE html>'.
            '<head>'.
			'<style>'.
				"@import url('https://fonts.googleapis.com/css?family=Cuprum|Prompt');".
		    	"@import url('https://fonts.googleapis.com/css?family=Martel+Sans');".
		        "
		        .tableLine {
		            background-color: #E6D7CF;
		        }
		        #tabela {
		            border-collapse: collapse;
		            border-spacing: 0;
		            width:100%;
		            height:100%;
		            margin:0px;
		            padding:0px;
		        }
		        body{
		            background-color: #E6D7CF;
		            height: 100%;
		        }
		        #tituloEmail {
		            font-family: 'Martel Sans', sans-serif;
		            font-size: 45px;
		            background-color: #a24e22;
		            color: #ffffff;
		        }
		        #emailBody {
		            padding-top: 5%;
		            font-family: 'Cuprum', sans-serif;
		            font-size: 25px;
		            text-align: center;
		            vertical-align: 25%;
		            padding-bottom: 5%;
		        }
		        #linhaTitulo {
		            height: 5px;
		            background-color: #b75826;
		        }".
			'</style>'.
			'<meta http-equiv="content-type" content="text/html; charset=UTF-8">'.
			'</head>'.
			'<body>'.
			'<div class="tabelaEmail">
	         <table id="tabela" >
	            <tr class="tableLine">
	               <td valign="top" align="center">
	                    <div id="tituloEmail">'.
	                      $subject
	                    .'</div>
	                    <div id="linhaTitulo"></div>
	               </td>
	            </tr>
	            <tr class="tableLine">
	               <td>
	                 <div id="emailBody">'.
	                    $content
	                 .'<br><img src="https://manage.teknisa.com/assets/images/logo/default.png?v" style="width: 250px;">
	             	 </div>
	               </td>
	            </tr>
	         </table>
	        </div>'.
			'</body>'.
			'</html>'
		;
	}
}