<?php
namespace Teknisa\MAN\Services;

use Doctrine\DBAL\Connection;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services\NovoCodigo;

class RelCampFideCashbackService {
	protected $entityManager;
	
	public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
	}
	

	public function buildParams ($conditions) {
	    $params = array (
            "DTINICIO" => $conditions["DTINICIO"],
            "DTFIM" => $conditions["DTFIM"],
            "CDFILIAL" => $conditions["CDFILIAL"] ?? "T",
            "CDCAMPANHA" => $conditions["CDCAMPANHA"] ?? "T",
            "CDCLIENTE" => $conditions["CDCLIENTE"] ?? "T",
            "CDCONSUMIDOR" => $conditions["CDCONSUMIDOR"] ?? "T",
            "CDCAIXA" => $conditions["CDCAIXA"] ?? "T",
            "NRSEQVENDA" => $conditions["NRSEQVENDA"] ?? "T"
        );
        
        return $params;
	}
	
	public function findSales ($conditions) {
	    $params = $this->buildParams($conditions);
	    return $this->entityManager->getConnection()->fetchAll(self::GET_SALES, $params);
	}
	
	
	public function findBonusGerado ($conditions) {
	    $params = $this->buildParams($conditions);
        
        return $this->entityManager->getConnection()->fetchAll(self::GET_BONUS_GERADO, $params);
	}
	
	public function findBonusResgatado ($conditions) {
	    $params = $this->buildParams($conditions);
        return $this->entityManager->getConnection()->fetchAll(self::GET_BONUS_RESGATADO, $params);
	}
	
	public function findBonusPerdido ($conditions) {
	    $params = $this->buildParams($conditions);
        
        return $this->entityManager->getConnection()->fetchAll(self::GET_BONUS_PERDIDO, $params);
	}
	
	 public function findBonusVencer ($conditions) {
	    $params = array (
            "DTINICIO" => $conditions["DTINICIO"],
            "DTFIM" => $conditions["DATAFIM"] ?? NULL,
            "CDFILIAL" => $conditions["CDFILIAL"] ?? "T",
            "CDCAMPANHA" => $conditions["CDCAMPANHA"] ?? "T",
            "CDCLIENTE" => $conditions["CDCLIENTE"] ?? "T",
            "CDCONSUMIDOR" => $conditions["CDCONSUMIDOR"] ?? "T",
            "CDCAIXA" => $conditions["CDCAIXA"] ?? "T",
            "NRSEQVENDA" => $conditions["NRSEQVENDA"] ?? "T"
        );

        return $this->entityManager->getConnection()->fetchAll(self::GET_BONUS_VENCER, $params);
	}
	
	 public function findBonusTotal ($conditions) {
	    $params = $this->buildParams($conditions);
        
        return $this->entityManager->getConnection()->fetchAll(self::GET_BONUS_CONS, $params);
	}
	
	public function findGeradoTotal ($conditions) {
	    $params = $this->buildParams($conditions);
	    
	    return $this->entityManager->getConnection()->fetchAssoc(self::GET_BONUS_GERADO_TOTAL, $params);
	}

	public function findResgatadoTotal ($conditions) {
        $params = $this->buildParams($conditions);
        
        return $this->entityManager->getConnection()->fetchAssoc(self::GET_BONUS_RESGATADO_TOTAL, $params);
    }
	
	public function findPerdidoTotal ($conditions) {
        $params = $this->buildParams($conditions);
        
        return $this->entityManager->getConnection()->fetchAssoc(self::GET_BONUS_PERDIDO_TOTAL, $params);
    }
    
	public function findVencerTotal ($conditions) {
	    $params = array (
            "DTINICIO" => $conditions["DTINICIO"],
            "DTFIM" => $conditions["DATAFIM"] ?? NULL,
            "CDFILIAL" => $conditions["CDFILIAL"] ?? "T",
            "CDCAMPANHA" => $conditions["CDCAMPANHA"] ?? "T",
            "CDCLIENTE" => $conditions["CDCLIENTE"] ?? "T",
            "CDCONSUMIDOR" => $conditions["CDCONSUMIDOR"] ?? "T",
            "CDCAIXA" => $conditions["CDCAIXA"] ?? "T",
            "NRSEQVENDA" => $conditions["NRSEQVENDA"] ?? "T"
        );

        return $this->entityManager->getConnection()->fetchAssoc(self::GET_BONUS_VENCER_TOTAL, $params);
    }
    
    public function findTotalConsumidor ($conditions) {
        $params = $this->buildParams($conditions);
        
        return $this->entityManager->getConnection()->fetchAssoc(self::GET_BONUS_TOTAL_CONS, $params);
    }
    
    const GET_SALES = "
        SELECT MPC.CDCAMPANHA as CAMPANHACD, MPC.DTMOVPONTOCONS as DATAVENDA, CONS.CDCLIENTE as CLIENTECD, (CONS.CDCONSUMIDOR || ' ' || CONS.NMCONSUMIDOR) AS CONSUMIDOR, V.VRTOTVENDA AS VRTOTALVENDA, MPC.VRMOVPONTOCONS, V.CDFILIAL, V.CDCAIXA as CAIXACD, V.NRSEQVENDA as NUMSEQVENDA 
            FROM MOVPONTOCONS MPC 
            INNER JOIN CAMPANHA C 
                ON C.CDCAMPANHA = MPC.CDCAMPANHA 
            INNER JOIN VENDA V 
                ON V.CDFILIAL = MPC.CDFILIAL 
                AND V.CDCAIXA = MPC.CDCAIXA 
                AND V.NRSEQVENDA = MPC.NRSEQVENDA 
            INNER JOIN CONSUMIDOR CONS
                ON CONS.CDCLIENTE = MPC.CDCLIENTE 
                AND CONS.CDCONSUMIDOR = MPC.CDCONSUMIDOR
        WHERE C.IDCAMPANHA = 'S'
                AND V.DTENTRVENDA BETWEEN :DTINICIO AND :DTFIM 
                AND ((V.CDFILIAL = :CDFILIAL) OR (:CDFILIAL = 'T'))
                AND ((MPC.CDCAMPANHA = :CDCAMPANHA) OR (:CDCAMPANHA = 'T'))
                AND ((CONS.CDCLIENTE = :CDCLIENTE) OR (:CDCLIENTE = 'T'))
                AND ((CONS.CDCONSUMIDOR = :CDCONSUMIDOR) OR (:CDCONSUMIDOR = 'T'))
                AND ((INSTR(:CDCAIXA,V.CDCAIXA)>0 AND :CDCAIXA <> 'T') OR (:CDCAIXA = 'T'))
        ORDER BY V.DTENTRVENDA DESC
    ";
    
    const GET_BONUS_GERADO = "
        SELECT
            MPC.CDCAMPANHA,
            TO_CHAR(MPC.DTMOVPONTOCONS, 'DD/MM/YYYY') as DTMOVPONTOCONS,
            CONS.CDCLIENTE, (CONS.CDCONSUMIDOR || ' ' || CONS.NMCONSUMIDOR) AS CONSUMIDOR,
            V.VRTOTVENDA AS VRTOTALVENDA,
            MPC.VRMOVPONTOCONS,
            V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA,
            F.NMFILIAL, C.DSCAMPANHA,
            TO_CHAR(LEAST(NVL(MPC.DTVALIDAPONTO, TO_DATE('31-12-9999', 'DD-MM-YYYY')),
                  NVL(C.DTFIMVIGCAMP, TO_DATE('31-12-9999', 'DD-MM-YYYY'))), 'DD/MM/YYYY') AS VALIDADE
        FROM MOVPONTOCONS MPC
            INNER JOIN CAMPANHA C ON C.CDCAMPANHA = MPC.CDCAMPANHA
            INNER JOIN VENDA V ON V.CDFILIAL = MPC.CDFILIAL AND V.CDCAIXA = MPC.CDCAIXA AND V.NRSEQVENDA = MPC.NRSEQVENDA
            INNER JOIN CONSUMIDOR CONS ON CONS.CDCLIENTE = MPC.CDCLIENTE AND CONS.CDCONSUMIDOR = MPC.CDCONSUMIDOR
            INNER JOIN FILIAL F ON V.CDFILIAL = F.CDFILIAL
        WHERE C.IDCAMPANHA = 'S'
            AND MPC.IDTPMOVCONS = 'C'
            AND V.IDSITUVENDA = 'O'
            AND ((TRUNC(MPC.DTVALIDAPONTO) >= :DTINICIO) OR (MPC.DTVALIDAPONTO IS NULL))
            AND TRUNC(V.DTENTRVENDA) BETWEEN :DTINICIO AND :DTFIM
            AND ((V.CDFILIAL = :CDFILIAL) OR (:CDFILIAL = 'T'))
            AND ((MPC.CDCAMPANHA = :CDCAMPANHA) OR (:CDCAMPANHA = 'T'))
            AND ((CONS.CDCLIENTE = :CDCLIENTE) OR (:CDCLIENTE = 'T'))
            AND ((CONS.CDCONSUMIDOR = :CDCONSUMIDOR) OR (:CDCONSUMIDOR = 'T'))
            AND ((INSTR(:CDCAIXA,V.CDCAIXA)>0 AND :CDCAIXA <> 'T') OR (:CDCAIXA = 'T'))
            AND ((INSTR(:NRSEQVENDA,V.NRSEQVENDA)>0 AND :NRSEQVENDA <> 'T') OR (:NRSEQVENDA = 'T'))
        ORDER BY V.DTENTRVENDA DESC
    ";
    
    const GET_BONUS_RESGATADO = "
        SELECT
            MPC.CDCAMPANHA,
            TO_CHAR(MPC.DTMOVPONTOCONS, 'DD/MM/YYYY') as DTMOVPONTOCONS,
            CONS.CDCLIENTE, (CONS.CDCONSUMIDOR || ' ' || CONS.NMCONSUMIDOR) AS CONSUMIDOR,
            V.VRTOTVENDA AS VRTOTALVENDA,
            MPC.VRMOVPONTOCONS,
            V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA,
            F.NMFILIAL, C.DSCAMPANHA,
            TO_CHAR(LEAST(NVL(MPC.DTVALIDAPONTO, TO_DATE('31-12-9999', 'DD-MM-YYYY')),
                  NVL(C.DTFIMVIGCAMP, TO_DATE('31-12-9999', 'DD-MM-YYYY'))), 'DD/MM/YYYY') AS VALIDADE
        FROM MOVPONTOCONS MPC
            INNER JOIN CAMPANHA C ON C.CDCAMPANHA = MPC.CDCAMPANHA
            INNER JOIN VENDA V ON V.CDFILIAL = MPC.CDFILIAL AND V.CDCAIXA = MPC.CDCAIXA AND V.NRSEQVENDA = MPC.NRSEQVENDA
            INNER JOIN CONSUMIDOR CONS ON CONS.CDCLIENTE = MPC.CDCLIENTE AND CONS.CDCONSUMIDOR = MPC.CDCONSUMIDOR
            INNER JOIN FILIAL F ON V.CDFILIAL = F.CDFILIAL
        WHERE C.IDCAMPANHA = 'S'
            AND MPC.IDTPMOVCONS = 'D'
            AND V.IDSITUVENDA = 'O'
            AND ((MPC.DTVALIDAPONTO >= :DTINICIO) OR (MPC.DTVALIDAPONTO IS NULL))
            AND V.DTENTRVENDA BETWEEN :DTINICIO AND :DTFIM
            AND ((V.CDFILIAL = :CDFILIAL) OR (:CDFILIAL = 'T'))
            AND ((MPC.CDCAMPANHA = :CDCAMPANHA) OR (:CDCAMPANHA = 'T'))
            AND ((CONS.CDCLIENTE = :CDCLIENTE) OR (:CDCLIENTE = 'T'))
            AND ((CONS.CDCONSUMIDOR = :CDCONSUMIDOR) OR (:CDCONSUMIDOR = 'T'))
            AND ((INSTR(:CDCAIXA,V.CDCAIXA)>0 AND :CDCAIXA <> 'T') OR (:CDCAIXA = 'T'))
            AND ((INSTR(:NRSEQVENDA,V.NRSEQVENDA)>0 AND :NRSEQVENDA <> 'T') OR (:NRSEQVENDA = 'T'))
        ORDER BY V.DTENTRVENDA DESC
    ";
    
    const GET_BONUS_PERDIDO = "
        SELECT
            MPC.CDCAMPANHA,
            TO_CHAR(MPC.DTMOVPONTOCONS, 'DD/MM/YYYY') as DTMOVPONTOCONS,
            CONS.CDCLIENTE, (CONS.CDCONSUMIDOR || ' ' || CONS.NMCONSUMIDOR) AS CONSUMIDOR,
            V.VRTOTVENDA AS VRTOTALVENDA,
            MPC.VRSALDOCONS,
            V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA,
            F.NMFILIAL, C.DSCAMPANHA,
            TO_CHAR(LEAST(NVL(MPC.DTVALIDAPONTO, TO_DATE('31-12-9999', 'DD-MM-YYYY')),
                  NVL(C.DTFIMVIGCAMP, TO_DATE('31-12-9999', 'DD-MM-YYYY'))), 'DD/MM/YYYY') AS VALIDADE
        FROM MOVPONTOCONS MPC
            INNER JOIN CAMPANHA C ON C.CDCAMPANHA = MPC.CDCAMPANHA
            INNER JOIN VENDA V ON V.CDFILIAL = MPC.CDFILIAL AND V.CDCAIXA = MPC.CDCAIXA AND V.NRSEQVENDA = MPC.NRSEQVENDA
            INNER JOIN CONSUMIDOR CONS ON CONS.CDCLIENTE = MPC.CDCLIENTE AND CONS.CDCONSUMIDOR = MPC.CDCONSUMIDOR
            INNER JOIN FILIAL F ON V.CDFILIAL = F.CDFILIAL
        WHERE C.IDCAMPANHA = 'S'
            AND MPC.IDTPMOVCONS = 'C'
            AND MPC.DTVALIDAPONTO < TRUNC(SYSDATE)
            AND V.DTENTRVENDA BETWEEN :DTINICIO AND :DTFIM
            AND ((V.CDFILIAL = :CDFILIAL) OR (:CDFILIAL = 'T'))
            AND ((MPC.CDCAMPANHA = :CDCAMPANHA) OR (:CDCAMPANHA = 'T'))
            AND ((CONS.CDCLIENTE = :CDCLIENTE) OR (:CDCLIENTE = 'T'))
            AND ((CONS.CDCONSUMIDOR = :CDCONSUMIDOR) OR (:CDCONSUMIDOR = 'T'))
            AND ((INSTR(:CDCAIXA,V.CDCAIXA)>0 AND :CDCAIXA <> 'T') OR (:CDCAIXA = 'T'))
            AND ((INSTR(:NRSEQVENDA,V.NRSEQVENDA)>0 AND :NRSEQVENDA <> 'T') OR (:NRSEQVENDA = 'T'))
        ORDER BY V.DTENTRVENDA DESC
    ";
    
    const GET_BONUS_VENCER = "
        SELECT
            MPC.CDCAMPANHA,
            TO_CHAR(MPC.DTVALIDAPONTO, 'DD/MM/YYYY') AS DTVALIDAPONTO,
            CONS.CDCLIENTE, (CONS.CDCONSUMIDOR || ' ' || CONS.NMCONSUMIDOR) AS CONSUMIDOR,
            MPC.VRSALDOCONS,
            V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA,
            F.NMFILIAL, C.DSCAMPANHA,
            TO_CHAR(LEAST(NVL(MPC.DTVALIDAPONTO, TO_DATE('31-12-9999', 'DD-MM-YYYY')),
                  NVL(C.DTFIMVIGCAMP, TO_DATE('31-12-9999', 'DD-MM-YYYY'))), 'DD/MM/YYYY') AS VALIDADE
        FROM MOVPONTOCONS MPC
            INNER JOIN CAMPANHA C ON C.CDCAMPANHA = MPC.CDCAMPANHA
            INNER JOIN VENDA V ON V.CDFILIAL = MPC.CDFILIAL AND V.CDCAIXA = MPC.CDCAIXA AND V.NRSEQVENDA = MPC.NRSEQVENDA
            INNER JOIN CONSUMIDOR CONS ON CONS.CDCLIENTE = MPC.CDCLIENTE AND CONS.CDCONSUMIDOR = MPC.CDCONSUMIDOR
            INNER JOIN FILIAL F ON V.CDFILIAL = F.CDFILIAL
        WHERE C.IDCAMPANHA = 'S'
            AND MPC.IDTPMOVCONS = 'C'
            AND ((:DTFIM IS NULL AND MPC.DTVALIDAPONTO BETWEEN TRUNC(SYSDATE) AND ADD_MONTHS(TRUNC(SYSDATE), 2))
                OR (MPC.DTVALIDAPONTO <= TRUNC(TO_DATE(:DTFIM, 'DD/MM/YYYY')))
                OR (MPC.DTVALIDAPONTO IS NULL))
            AND ((V.CDFILIAL = :CDFILIAL) OR (:CDFILIAL = 'T'))
            AND ((MPC.CDCAMPANHA = :CDCAMPANHA) OR (:CDCAMPANHA = 'T'))
            AND ((CONS.CDCLIENTE = :CDCLIENTE) OR (:CDCLIENTE = 'T'))
            AND ((CONS.CDCONSUMIDOR = :CDCONSUMIDOR) OR (:CDCONSUMIDOR = 'T'))
            AND ((INSTR(:CDCAIXA,V.CDCAIXA)>0 AND :CDCAIXA <> 'T') OR (:CDCAIXA = 'T'))
            AND ((INSTR(:NRSEQVENDA,V.NRSEQVENDA)>0 AND :NRSEQVENDA <> 'T') OR (:NRSEQVENDA = 'T'))
        ORDER BY V.DTENTRVENDA DESC
    ";
    
    const GET_BONUS_CONS = "
        SELECT 
            C.CDCAMPANHA,
            C.DSCAMPANHA,
            TO_CHAR(MPC.DTVALIDAPONTO, 'DD/MM/YYYY') AS DTVALIDAPONTO,
            CONS.CDCLIENTE,
            CONS.CDCONSUMIDOR,
            CONS.NMCONSUMIDOR AS CONSUMIDOR,
            SUM(DECODE(V.IDSITUVENDA,'O',V.VRTOTVENDA,0)) AS VRTOTALGASTO,
            SUM(MPC.VRSALDOCONS) AS VRSALDOCONS,
            V.CDFILIAL, F.NMFILIAL,
            V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA,
            TO_CHAR(LEAST(NVL(MPC.DTVALIDAPONTO, TO_DATE('31-12-9999', 'DD-MM-YYYY')),
                  NVL(C.DTFIMVIGCAMP, TO_DATE('31-12-9999', 'DD-MM-YYYY'))), 'DD/MM/YYYY') AS VALIDADE
        FROM 
            MOVPONTOCONS MPC
            INNER JOIN CAMPANHA C ON C.CDCAMPANHA = MPC.CDCAMPANHA
            INNER JOIN VENDA V ON V.CDFILIAL = MPC.CDFILIAL AND V.CDCAIXA = MPC.CDCAIXA AND V.NRSEQVENDA = MPC.NRSEQVENDA
            INNER JOIN CONSUMIDOR CONS ON CONS.CDCLIENTE = MPC.CDCLIENTE AND CONS.CDCONSUMIDOR = MPC.CDCONSUMIDOR
            INNER JOIN FILIAL F ON V.CDFILIAL = F.CDFILIAL
        WHERE 
            C.IDCAMPANHA = 'S'
            AND MPC.IDTPMOVCONS = 'C'
            AND ((MPC.DTVALIDAPONTO >= TRUNC(SYSDATE)) OR (MPC.DTVALIDAPONTO IS NULL))
            AND V.DTENTRVENDA BETWEEN C.DTINIVIGCAMP AND C.DTFIMVIGCAMP
            AND ((V.CDFILIAL = :CDFILIAL) OR (:CDFILIAL = 'T'))
            AND ((MPC.CDCAMPANHA = :CDCAMPANHA) OR (:CDCAMPANHA = 'T'))
            AND ((CONS.CDCLIENTE = :CDCLIENTE) OR (:CDCLIENTE = 'T'))
            AND ((CONS.CDCONSUMIDOR = :CDCONSUMIDOR) OR (:CDCONSUMIDOR = 'T'))
            AND ((INSTR(:CDCAIXA,V.CDCAIXA)>0 AND :CDCAIXA <> 'T') OR (:CDCAIXA = 'T'))
            AND ((INSTR(:NRSEQVENDA,V.NRSEQVENDA)>0 AND :NRSEQVENDA <> 'T') OR (:NRSEQVENDA = 'T'))
        GROUP BY 
            C.CDCAMPANHA, C.DSCAMPANHA, CONS.CDCLIENTE, CONS.CDCONSUMIDOR, CONS.NMCONSUMIDOR, LEAST(NVL(MPC.DTVALIDAPONTO, TO_DATE('31-12-9999', 'DD-MM-YYYY')), NVL(C.DTFIMVIGCAMP, TO_DATE('31-12-9999', 'DD-MM-YYYY'))), 
            V.CDFILIAL, F.NMFILIAL, V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, MPC.DTVALIDAPONTO
        ORDER BY
            VRTOTALGASTO DESC, VRSALDOCONS DESC
    ";
    
    const GET_BONUS_GERADO_TOTAL = "
        SELECT SUM(MPC.VRMOVPONTOCONS) AS TOTALMOVPONTOCONS, COUNT(DISTINCT MPC.CDFILIAL||MPC.CDCAIXA||MPC.NRSEQVENDA) AS NRTICKETS
            FROM MOVPONTOCONS MPC
            INNER JOIN CAMPANHA C ON C.CDCAMPANHA = MPC.CDCAMPANHA
            INNER JOIN VENDA V ON V.CDFILIAL = MPC.CDFILIAL AND V.CDCAIXA = MPC.CDCAIXA AND V.NRSEQVENDA = MPC.NRSEQVENDA
            INNER JOIN CONSUMIDOR CONS ON CONS.CDCLIENTE = MPC.CDCLIENTE AND CONS.CDCONSUMIDOR = MPC.CDCONSUMIDOR
        WHERE C.IDCAMPANHA = 'S'
            AND MPC.IDTPMOVCONS = 'C'
            AND V.IDSITUVENDA = 'O'
            AND ((MPC.DTVALIDAPONTO >= :DTINICIO) OR (MPC.DTVALIDAPONTO IS NULL))
            AND V.DTENTRVENDA BETWEEN :DTINICIO AND :DTFIM
            AND ((V.CDFILIAL = :CDFILIAL) OR (:CDFILIAL = 'T'))
            AND ((MPC.CDCAMPANHA = :CDCAMPANHA) OR (:CDCAMPANHA = 'T'))
            AND ((CONS.CDCLIENTE = :CDCLIENTE) OR (:CDCLIENTE = 'T'))
            AND ((CONS.CDCONSUMIDOR = :CDCONSUMIDOR) OR (:CDCONSUMIDOR = 'T'))
            AND ((INSTR(:CDCAIXA,V.CDCAIXA)>0 AND :CDCAIXA <> 'T') OR (:CDCAIXA = 'T'))
            AND ((INSTR(:NRSEQVENDA,V.NRSEQVENDA)>0 AND :NRSEQVENDA <> 'T') OR (:NRSEQVENDA = 'T'))
        ORDER BY V.DTENTRVENDA DESC
    ";
    
    const GET_BONUS_RESGATADO_TOTAL = "
        SELECT SUM(MPC.VRMOVPONTOCONS) AS TOTALMOVPONTOCONS, COUNT(DISTINCT MPC.CDFILIAL||MPC.CDCAIXA||MPC.NRSEQVENDA) AS NRTICKETS
            FROM MOVPONTOCONS MPC
            INNER JOIN CAMPANHA C ON C.CDCAMPANHA = MPC.CDCAMPANHA
            INNER JOIN VENDA V ON V.CDFILIAL = MPC.CDFILIAL AND V.CDCAIXA = MPC.CDCAIXA AND V.NRSEQVENDA = MPC.NRSEQVENDA
            INNER JOIN CONSUMIDOR CONS ON CONS.CDCLIENTE = MPC.CDCLIENTE AND CONS.CDCONSUMIDOR = MPC.CDCONSUMIDOR
        WHERE C.IDCAMPANHA = 'S'
            AND MPC.IDTPMOVCONS = 'D'
            AND V.IDSITUVENDA = 'O'
            AND ((MPC.DTVALIDAPONTO >= :DTINICIO) OR (MPC.DTVALIDAPONTO IS NULL))
            AND V.DTENTRVENDA BETWEEN :DTINICIO AND :DTFIM
            AND ((V.CDFILIAL = :CDFILIAL) OR (:CDFILIAL = 'T'))
            AND ((MPC.CDCAMPANHA = :CDCAMPANHA) OR (:CDCAMPANHA = 'T'))
            AND ((CONS.CDCLIENTE = :CDCLIENTE) OR (:CDCLIENTE = 'T'))
            AND ((CONS.CDCONSUMIDOR = :CDCONSUMIDOR) OR (:CDCONSUMIDOR = 'T'))
            AND ((INSTR(:CDCAIXA,V.CDCAIXA)>0 AND :CDCAIXA <> 'T') OR (:CDCAIXA = 'T'))
            AND ((INSTR(:NRSEQVENDA,V.NRSEQVENDA)>0 AND :NRSEQVENDA <> 'T') OR (:NRSEQVENDA = 'T'))
        ORDER BY V.DTENTRVENDA DESC
    ";
    
    const GET_BONUS_PERDIDO_TOTAL = "
        SELECT NVL(SUM(MPC.VRSALDOCONS),0) AS TOTALMOVPONTOCONS, COUNT(DISTINCT MPC.CDFILIAL||MPC.CDCAIXA||MPC.NRSEQVENDA) AS NRTICKETS
            FROM MOVPONTOCONS MPC
            INNER JOIN CAMPANHA C ON C.CDCAMPANHA = MPC.CDCAMPANHA
            INNER JOIN VENDA V ON V.CDFILIAL = MPC.CDFILIAL AND V.CDCAIXA = MPC.CDCAIXA AND V.NRSEQVENDA = MPC.NRSEQVENDA
            INNER JOIN CONSUMIDOR CONS ON CONS.CDCLIENTE = MPC.CDCLIENTE AND CONS.CDCONSUMIDOR = MPC.CDCONSUMIDOR
        WHERE C.IDCAMPANHA = 'S'
            AND MPC.IDTPMOVCONS = 'C'
            AND MPC.DTVALIDAPONTO < TRUNC(SYSDATE)
            AND V.DTENTRVENDA BETWEEN :DTINICIO AND :DTFIM
            AND ((V.CDFILIAL = :CDFILIAL) OR (:CDFILIAL = 'T'))
            AND ((MPC.CDCAMPANHA = :CDCAMPANHA) OR (:CDCAMPANHA = 'T'))
            AND ((CONS.CDCLIENTE = :CDCLIENTE) OR (:CDCLIENTE = 'T'))
            AND ((CONS.CDCONSUMIDOR = :CDCONSUMIDOR) OR (:CDCONSUMIDOR = 'T'))
            AND ((INSTR(:CDCAIXA,V.CDCAIXA)>0 AND :CDCAIXA <> 'T') OR (:CDCAIXA = 'T'))
            AND ((INSTR(:NRSEQVENDA,V.NRSEQVENDA)>0 AND :NRSEQVENDA <> 'T') OR (:NRSEQVENDA = 'T'))
        ORDER BY V.DTENTRVENDA DESC
    ";
    
    const GET_BONUS_VENCER_TOTAL = "
        SELECT SUM(MPC.VRSALDOCONS) AS TOTALMOVPONTOCONS, COUNT(DISTINCT MPC.CDFILIAL||MPC.CDCAIXA||MPC.NRSEQVENDA) AS NRTICKETS
            FROM MOVPONTOCONS MPC
            INNER JOIN CAMPANHA C ON C.CDCAMPANHA = MPC.CDCAMPANHA
            INNER JOIN VENDA V ON V.CDFILIAL = MPC.CDFILIAL AND V.CDCAIXA = MPC.CDCAIXA AND V.NRSEQVENDA = MPC.NRSEQVENDA
            INNER JOIN CONSUMIDOR CONS ON CONS.CDCLIENTE = MPC.CDCLIENTE AND CONS.CDCONSUMIDOR = MPC.CDCONSUMIDOR
        WHERE C.IDCAMPANHA = 'S'
            AND MPC.IDTPMOVCONS = 'C'
            AND ((:DTFIM IS NULL AND MPC.DTVALIDAPONTO BETWEEN TRUNC(SYSDATE) AND ADD_MONTHS(TRUNC(SYSDATE), 2))
                OR (MPC.DTVALIDAPONTO <= TRUNC(TO_DATE(:DTFIM, 'DD/MM/YYYY')))
                OR (MPC.DTVALIDAPONTO IS NULL))
            AND ((V.CDFILIAL = :CDFILIAL) OR (:CDFILIAL = 'T'))
            AND ((MPC.CDCAMPANHA = :CDCAMPANHA) OR (:CDCAMPANHA = 'T'))
            AND ((CONS.CDCLIENTE = :CDCLIENTE) OR (:CDCLIENTE = 'T'))
            AND ((CONS.CDCONSUMIDOR = :CDCONSUMIDOR) OR (:CDCONSUMIDOR = 'T'))
            AND ((INSTR(:CDCAIXA,V.CDCAIXA)>0 AND :CDCAIXA <> 'T') OR (:CDCAIXA = 'T'))
            AND ((INSTR(:NRSEQVENDA,V.NRSEQVENDA)>0 AND :NRSEQVENDA <> 'T') OR (:NRSEQVENDA = 'T'))
        ORDER BY V.DTENTRVENDA DESC
    ";
    
    const GET_BONUS_TOTAL_CONS = "
        SELECT 
            SUM(DECODE(V.IDSITUVENDA,'O',V.VRTOTVENDA,0)) AS VRTOTALGASTO,
            SUM(MPC.VRSALDOCONS) AS VRSALDOCONS
        FROM 
            MOVPONTOCONS MPC
            INNER JOIN CAMPANHA C ON C.CDCAMPANHA = MPC.CDCAMPANHA
            INNER JOIN VENDA V ON V.CDFILIAL = MPC.CDFILIAL AND V.CDCAIXA = MPC.CDCAIXA AND V.NRSEQVENDA = MPC.NRSEQVENDA
            INNER JOIN CONSUMIDOR CONS ON CONS.CDCLIENTE = MPC.CDCLIENTE AND CONS.CDCONSUMIDOR = MPC.CDCONSUMIDOR
        WHERE 
            C.IDCAMPANHA = 'S'
            AND MPC.IDTPMOVCONS = 'C'
            AND ((MPC.DTVALIDAPONTO >= TRUNC(SYSDATE)) OR (MPC.DTVALIDAPONTO IS NULL))
            AND V.DTENTRVENDA BETWEEN C.DTINIVIGCAMP AND C.DTFIMVIGCAMP
            AND ((V.CDFILIAL = :CDFILIAL) OR (:CDFILIAL = 'T'))
            AND ((MPC.CDCAMPANHA = :CDCAMPANHA) OR (:CDCAMPANHA = 'T'))
            AND ((CONS.CDCLIENTE = :CDCLIENTE) OR (:CDCLIENTE = 'T'))
            AND ((CONS.CDCONSUMIDOR = :CDCONSUMIDOR) OR (:CDCONSUMIDOR = 'T'))
            AND ((INSTR(:CDCAIXA,V.CDCAIXA)>0 AND :CDCAIXA <> 'T') OR (:CDCAIXA = 'T'))
            AND ((INSTR(:NRSEQVENDA,V.NRSEQVENDA)>0 AND :NRSEQVENDA <> 'T') OR (:NRSEQVENDA = 'T'))
        ORDER BY
            VRTOTALGASTO DESC, VRSALDOCONS DESC
    ";
}